# mypy: ignore-errors
from google.protobuf.message_factory import GetMessageClass
from google.protobuf.descriptor import MethodDescriptor
from google.protobuf.json_format import MessageToDict, ParseDict

from typing import Any, Dict, Iterator, List, Union


class ReflectedMethod:
    def __init__(self, channel, method_desc: MethodDescriptor):
        """
        Build a method handler for a given method full name
        """
        self.input_prototype = GetMessageClass(method_desc.input_type)
        self.output_prototype = GetMessageClass(method_desc.output_type)

        if method_desc.client_streaming and method_desc.server_streaming:
            abstract_handler = channel.stream_stream
            req_parser = self.parse_stream_request
            res_parser = self.parse_stream_response
        elif method_desc.client_streaming:
            abstract_handler = channel.stream_unary
            req_parser = self.parse_stream_request
            res_parser = self.parse_unary_response
        elif method_desc.server_streaming:
            abstract_handler = channel.unary_stream
            req_parser = self.parse_unary_request
            res_parser = self.parse_stream_response
        else:
            abstract_handler = channel.unary_unary
            req_parser = self.parse_unary_request
            res_parser = self.parse_unary_response

        self.handler = abstract_handler(
            f"/{method_desc.containing_service.full_name}/{method_desc.name}",
            self.input_prototype.SerializeToString,
            self.output_prototype.FromString,
        )

        self.request_parser = req_parser
        self.response_parser = res_parser

    def parse_stream_request(
        self, json_request_iter: Iterator, options
    ) -> Iterator[Dict[str, Any]]:
        requests = []
        for request in json_request_iter:
            requests.append(ParseDict(request, self.input_prototype(), **options))
        return iter(requests)

    def parse_unary_request(self, json_request: Dict[str, Any], options) -> Any:
        return ParseDict(json_request, self.input_prototype(), **options)

    def parse_stream_response(
        self, response_iter: Iterator, options
    ) -> List[Dict[str, Any]]:
        responses = []
        for response in response_iter:
            responses.append(MessageToDict(response, **options))
        return responses

    def parse_unary_response(self, response: Any, options) -> Dict[str, Any]:
        return MessageToDict(response, **options)

    def request(
        self, requests: Union[Iterator, Dict[str, Any]], options={}
    ) -> Union[List[Dict[str, Any]], Dict[str, Any]]:
        """
        Make a request to a given service and method with a given request
        :param json_request: Request as a dict
        :return: Response as a dict
        """

        return self.response_parser(
            (self.handler(self.request_parser(requests, options), timeout=10)), options
        )



import grpc
import logging

from .reflected_method import ReflectedMethod

from google.protobuf.descriptor_pool import DescriptorPool
from grpc_reflection.v1alpha.proto_reflection_descriptor_database import (
    ProtoReflectionDescriptorDatabase,
)
from typing import Any, Dict, List, Optional, Union, Iterator


class Client:
    logger = logging.getLogger(__name__)

    def __init__(
        self,
        address: str,
        secure_channel: bool = False,
        credentials: Optional[grpc.ChannelCredentials] = None,
    ):
        """
        Initialize a grpc client to a reflection enabled server.
        Methods are discovered at runtime using reflection.
        :param address: Address of the server
        :param secure_channel: Whether to use a secure channel
        :param credentials: Credentials to use for secure channel
        """
        if secure_channel:
            if credentials is None:
                raise Exception("Secure channel requested but no credentials provided")
            self.channel = grpc.secure_channel(address, credentials)
        else:
            self.channel = grpc.insecure_channel(address)
        self.reflection_db = ProtoReflectionDescriptorDatabase(self.channel)
        self.descriptor_pool = DescriptorPool(self.reflection_db)
        self.reflected_methods = {}
        for service in self.reflection_db.get_services():
            for method_full_name in self.get_method_full_names(service):
                method_desc = self.descriptor_pool.FindMethodByName(method_full_name)
                self.reflected_methods[method_full_name] = ReflectedMethod(
                    self.channel, method_desc
                )

    def get_services(self) -> List[str]:
        """
        List the names of available services.
        :return: List of service names as strings
        """
        return self.reflection_db.get_services()

    def get_method_names(self, service: str) -> List[str]:
        """
        List of method names for a given named service
        :param service: Service name as string
        :return: List of method names as strings
        """
        if service not in self.get_services():
            raise ValueError(f"Service {service} not available")

        return [
            method.name
            for method in self.descriptor_pool.FindServiceByName(service).methods
        ]

    def get_method_full_names(self, service: str) -> List[str]:
        """
        List of method full names for a given named service
        :param service: Service name as string
        :return: List of method full names as strings
        """
        if service not in self.get_services():
            raise ValueError(f"Service {service} not available")

        return [
            method.full_name
            for method in self.descriptor_pool.FindServiceByName(service).methods
        ]

    def request(
        self, service: str, method_name: str, request: Union[Iterator, Dict[str, Any]]
    ) -> Union[List[Dict[str, Any]], Dict[str, Any]]:
        """
        Make a request to a given service and method with a given request
        :param service: Service name as string
        :param method_name: Method name as string
        :param json_request: Request as a dict
        :return: Response as a dict
        """

        reflected_method = self.reflected_methods[f"{service}.{method_name}"]

        # Make request
        return reflected_method.request(request, {})



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class TestRequest(_message.Message):
    __slots__ = ["factor", "readings", "uuid", "sample_flag", "request_name", "extra_data"]
    FACTOR_FIELD_NUMBER: _ClassVar[int]
    READINGS_FIELD_NUMBER: _ClassVar[int]
    UUID_FIELD_NUMBER: _ClassVar[int]
    SAMPLE_FLAG_FIELD_NUMBER: _ClassVar[int]
    REQUEST_NAME_FIELD_NUMBER: _ClassVar[int]
    EXTRA_DATA_FIELD_NUMBER: _ClassVar[int]
    factor: int
    readings: _containers.RepeatedScalarFieldContainer[float]
    uuid: int
    sample_flag: bool
    request_name: str
    extra_data: _containers.RepeatedScalarFieldContainer[bytes]
    def __init__(self, factor: _Optional[int] = ..., readings: _Optional[_Iterable[float]] = ..., uuid: _Optional[int] = ..., sample_flag: bool = ..., request_name: _Optional[str] = ..., extra_data: _Optional[_Iterable[bytes]] = ...) -> None: ...

class TestResponse(_message.Message):
    __slots__ = ["average", "feedback"]
    AVERAGE_FIELD_NUMBER: _ClassVar[int]
    FEEDBACK_FIELD_NUMBER: _ClassVar[int]
    average: float
    feedback: str
    def __init__(self, average: _Optional[float] = ..., feedback: _Optional[str] = ...) -> None: ...



import multiprocessing
import pytest
import time

from servers.client_tester_server import ClientTesterServer


def client_tester_server_starter():
    server = ClientTesterServer('50051')
    server.serve()

@pytest.fixture(scope="session", autouse=True)
def client_tester_server():
    client_tester_server_process = multiprocessing.Process(target=client_tester_server_starter)
    client_tester_server_process.start()
    time.sleep(1)
    yield
    client_tester_server_process.terminate()



from concurrent import futures
from grpc_reflection.v1alpha import reflection

import grpc
import logging
from .client_tester_pb2_grpc import ClientTesterServicer, add_ClientTesterServicer_to_server
from .client_tester_pb2 import TestResponse, DESCRIPTOR

class ClientTester(ClientTesterServicer):
    """
    Logical methods to generate test responses for the client tester.
    """

    def TestUnaryUnary(self, request, context):
        return TestResponse(average=0.0, feedback="Acceptable")

    def TestUnaryStream(self, request, context):
        for reading in request.readings:
            yield TestResponse(average=0.0, feedback="Acceptable")

    def TestStreamUnary(self, request_iterator, context):
        for request in request_iterator:
            pass
        return TestResponse(average=0.0, feedback="Acceptable")

    def TestStreamStream(self, request_iterator, context):
        for request in request_iterator:
            for reading in request.readings:
                yield TestResponse(average=0.0, feedback="Acceptable")


class ClientTesterServer():
    """
    Simple Reflection Enabled GRPC Server for the purposes of testing a
    GRPC reflection client.
    """

    server = None

    def __init__(self, port: str):
        self.server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        add_ClientTesterServicer_to_server(ClientTester(), self.server)
        SERVICE_NAMES = (
            DESCRIPTOR.services_by_name['ClientTester'].full_name,
            reflection.SERVICE_NAME,
        )
        reflection.enable_server_reflection(SERVICE_NAMES, self.server)
        self.server.add_insecure_port(f'[::]:{port}')

    def serve(self):
        logging.debug('Server starting...')
        self.server.start()
        logging.debug('Server running...')
        self.server.wait_for_termination()

    def shutdown(self):
        self.server.stop(grace=3)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.shutdown()


if __name__ == "__main__":
    server = ClientTesterServer("50051")
    server.serve()



import pytest

from rgrpc.client import Client

@pytest.fixture(scope="session", autouse=True)
def rgrpc_client():
    try:
        yield Client('localhost:50051')
    except Exception as e:
        pytest.fail(f"Failed to initialize rgrpc client: {e}")


def test_get_services(rgrpc_client):
    services = rgrpc_client.get_services()
    assert len(services) == 2
    assert 'client_tester.ClientTester' in services
    assert 'grpc.reflection.v1alpha.ServerReflection' in services

def test_get_method_names(rgrpc_client):
    methods = rgrpc_client.get_method_names('client_tester.ClientTester')
    assert len(methods) == 4
    assert 'TestUnaryUnary' in methods
    assert 'TestUnaryStream' in methods
    assert 'TestStreamUnary' in methods
    assert 'TestStreamStream' in methods

def test_get_method_full_names(rgrpc_client):
    methods = rgrpc_client.get_method_full_names('client_tester.ClientTester')
    assert len(methods) == 4
    assert 'client_tester.ClientTester.TestUnaryUnary' in methods
    assert 'client_tester.ClientTester.TestUnaryStream' in methods
    assert 'client_tester.ClientTester.TestStreamUnary' in methods
    assert 'client_tester.ClientTester.TestStreamStream' in methods

def test_make_unary_unary_request(rgrpc_client):
    response = rgrpc_client.request(
        "client_tester.ClientTester",
        "TestUnaryUnary",
        {"factor": 1.0, "readings": [1.0, 2.0, 3.0]}
    )
    assert response == {'feedback': 'Acceptable'}

def test_make_unary_stream_request(rgrpc_client):
    responses = rgrpc_client.request(
        "client_tester.ClientTester",
        "TestUnaryStream",
        {"factor": 1.0, "readings": [1.0, 2.0, 3.0]}
    )
    for response in responses:
        assert response == {'feedback': 'Acceptable'}

def test_make_stream_unary_request(rgrpc_client):
    factors = [1.0, 2.0, 3.0]
    readings = [1.0, 2.0, 3.0]
    response = rgrpc_client.request(
        "client_tester.ClientTester","TestStreamUnary",
        iter([{"factor": factor, "readings": readings} for factor in factors])
    )
    assert response == {'feedback': 'Acceptable'}

def test_make_stream_stream_request(rgrpc_client):
    factors = [1.0, 2.0, 3.0]
    readings = [1.0, 2.0, 3.0]
    responses = rgrpc_client.request(
        "client_tester.ClientTester","TestStreamStream",
        iter([{"factor": factor, "readings": readings} for factor in factors])
    )
    for response in responses:
        assert response == {'feedback': 'Acceptable'}



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client_tester.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x63lient_tester.proto\x12\rclient_tester\"|\n\x0bTestRequest\x12\x0e\n\x06\x66\x61\x63tor\x18\x01 \x01(\x05\x12\x10\n\x08readings\x18\x02 \x03(\x02\x12\x0c\n\x04uuid\x18\x03 \x01(\x04\x12\x13\n\x0bsample_flag\x18\x04 \x01(\x08\x12\x14\n\x0crequest_name\x18\x05 \x01(\t\x12\x12\n\nextra_data\x18\x06 \x03(\x0c\"1\n\x0cTestResponse\x12\x0f\n\x07\x61verage\x18\x01 \x01(\x01\x12\x10\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\x02 \x01(\t2\xce\x02\n\x0c\x43lientTester\x12K\n\x0eTestUnaryUnary\x12\x1a.client_tester.TestRequest\x1a\x1b.client_tester.TestResponse\"\x00\x12N\n\x0fTestUnaryStream\x12\x1a.client_tester.TestRequest\x1a\x1b.client_tester.TestResponse\"\x00\x30\x01\x12N\n\x0fTestStreamUnary\x12\x1a.client_tester.TestRequest\x1a\x1b.client_tester.TestResponse\"\x00(\x01\x12Q\n\x10TestStreamStream\x12\x1a.client_tester.TestRequest\x1a\x1b.client_tester.TestResponse\"\x00(\x01\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'client_tester_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_TESTREQUEST']._serialized_start=38
  _globals['_TESTREQUEST']._serialized_end=162
  _globals['_TESTRESPONSE']._serialized_start=164
  _globals['_TESTRESPONSE']._serialized_end=213
  _globals['_CLIENTTESTER']._serialized_start=216
  _globals['_CLIENTTESTER']._serialized_end=550
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import client_tester_pb2 as client__tester__pb2


class ClientTesterStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.TestUnaryUnary = channel.unary_unary(
                '/client_tester.ClientTester/TestUnaryUnary',
                request_serializer=client__tester__pb2.TestRequest.SerializeToString,
                response_deserializer=client__tester__pb2.TestResponse.FromString,
                )
        self.TestUnaryStream = channel.unary_stream(
                '/client_tester.ClientTester/TestUnaryStream',
                request_serializer=client__tester__pb2.TestRequest.SerializeToString,
                response_deserializer=client__tester__pb2.TestResponse.FromString,
                )
        self.TestStreamUnary = channel.stream_unary(
                '/client_tester.ClientTester/TestStreamUnary',
                request_serializer=client__tester__pb2.TestRequest.SerializeToString,
                response_deserializer=client__tester__pb2.TestResponse.FromString,
                )
        self.TestStreamStream = channel.stream_stream(
                '/client_tester.ClientTester/TestStreamStream',
                request_serializer=client__tester__pb2.TestRequest.SerializeToString,
                response_deserializer=client__tester__pb2.TestResponse.FromString,
                )


class ClientTesterServicer(object):
    """Missing associated documentation comment in .proto file."""

    def TestUnaryUnary(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TestUnaryStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TestStreamUnary(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TestStreamStream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ClientTesterServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'TestUnaryUnary': grpc.unary_unary_rpc_method_handler(
                    servicer.TestUnaryUnary,
                    request_deserializer=client__tester__pb2.TestRequest.FromString,
                    response_serializer=client__tester__pb2.TestResponse.SerializeToString,
            ),
            'TestUnaryStream': grpc.unary_stream_rpc_method_handler(
                    servicer.TestUnaryStream,
                    request_deserializer=client__tester__pb2.TestRequest.FromString,
                    response_serializer=client__tester__pb2.TestResponse.SerializeToString,
            ),
            'TestStreamUnary': grpc.stream_unary_rpc_method_handler(
                    servicer.TestStreamUnary,
                    request_deserializer=client__tester__pb2.TestRequest.FromString,
                    response_serializer=client__tester__pb2.TestResponse.SerializeToString,
            ),
            'TestStreamStream': grpc.stream_stream_rpc_method_handler(
                    servicer.TestStreamStream,
                    request_deserializer=client__tester__pb2.TestRequest.FromString,
                    response_serializer=client__tester__pb2.TestResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'client_tester.ClientTester', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ClientTester(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def TestUnaryUnary(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/client_tester.ClientTester/TestUnaryUnary',
            client__tester__pb2.TestRequest.SerializeToString,
            client__tester__pb2.TestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TestUnaryStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/client_tester.ClientTester/TestUnaryStream',
            client__tester__pb2.TestRequest.SerializeToString,
            client__tester__pb2.TestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TestStreamUnary(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/client_tester.ClientTester/TestStreamUnary',
            client__tester__pb2.TestRequest.SerializeToString,
            client__tester__pb2.TestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TestStreamStream(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/client_tester.ClientTester/TestStreamStream',
            client__tester__pb2.TestRequest.SerializeToString,
            client__tester__pb2.TestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
